/* Header for main routine                                                   */

#undef EXT
#undef INITIALIZE

#ifdef DRIVER
#define EXT
#define INITIALIZE
#undef DRIVER
#else
#define EXT extern
#endif

/*---------------------------------------------------------------------------*/
/* Microsoft C include files needed by the stuff in this file                */

#include <setjmp.h>
#include <time.h>

/*---------------------------------------------------------------------------*/
/* Blaise C Tools Plus 5.0 include files                                     */

#include <bfiles.h>

/*---------------------------------------------------------------------------*/
/* Global constants                                                          */

#define DRIVERID 13                    /* driver version number x 10         */

#define MAXENGINES 256                 /* most engines in the array          */
#define ARRAYTIMEOUT 1.0               /* longest delay before response      */

#define FINEZSTEP    1                 /* pel step during zooming            */
#define COARSEZSTEP 16                 /*   ... with ctrl held down          */

#define NUMLEN 8                       /* bytes per extended precision num   */
#define DBLPREC 16                     /* total significant digits in double */


/*---------------------------------------------------------------------------*/
/* Global structures and typedefs                                            */


/*--- EXTMATH.C                                                              */
/* Extended precision numbers look like this:                                */
/*  + 0 0 0 . 0 0 0 0 | 0 0 0 0 0 0 0 0 | NUMLEN-3 bytes | 0 0 0 0 0 0 0 0   */
/*  ^ sign  ^ binary point                                                   */
/* Negative numbers are in two's complement form for simplicity              */
/* The overall length is 64 bits, of which 60 are fractional                 */
/* Overflow occurs for positive numbers > 7.999 and negative < -8.000        */
/*  but we blithely ignore all that...                                       */
/* For multiplies, 2.8 is about the biggest thing with a useful square...    */
/*  so there's some motivation to restrict the input values!                 */
/* Extended precision numbers are stored with least significant byte first,  */
/*  in standard Intel format...                                              */
/*  this is exactly backwards from the way the engine handles them, natch    */


typedef unsigned char EXTNUM[NUMLEN];


/* Bit definitions for "double" variables                                    */
/* High two bytes include bit field to cover sign and exponent               */
/*  ... remember that these are stored with MSB in highest addressed byte    */

struct dbits {
  unsigned char mantissa[6];           /* mantissa[0] is lowest order byte   */
  unsigned int highmant:4;             /* high order bits of mantissa        */
  unsigned int exponent:11;            /* excess-1023 exponent               */
  unsigned int signbit:1;              /* sign for sign-mag mantissa         */
};

typedef union {                        /* map double & bits to same spot     */
  double dbl;
  struct dbits bits;
} DOUBLEBITS;


/*--- ZPLANE.C                                                               */

typedef struct {                       /* points or distances in Z plane     */
  double r;
  double i;
} COMPLEX;


typedef struct {                       /* points or distances on screen      */
  int x;
  int y;
  int color;
} SCREENPT;


typedef struct {                       /* points or distances on either      */
  SCREENPT scrn;
  COMPLEX  cplx;
} DATUM;



/*--- SCREEN.C                                                               */

#define GC_DRAW 0x00                   /* Graphics Controller Modes          */
#define GC_AND  0x08                   /*   used by line drawing routines    */
#define GC_OR   0x10
#define GC_XOR  0x18

#define POOF 4                         /* split screen scrolling rate        */
#define FAST 2
#define NORMAL 1

#define INSTANT  0.001                 /* delay before splitting screen      */
#define TEASER   1.0                   /*   negative -> press any key        */
#define READABLE 3.0
#define PROTRACT 5.0

#define MAXTEXT 160                    /* most characters per line           */
#define HRATE 8                        /* starting horizontal scroll rate    */
#define VRATE 8                        /* starting vertical   scroll rate    */

/*--- SCRNFILE.C                                                             */

#define DESCRIPTORSIZE 512             /* bytes in image descriptor          */

struct descflds {
  long int ProgID;                     /* program version identifier         */
  unsigned int UseVidMode;             /* video mode in effect during save   */
  long int PassesPerRow;               /* number of passes per screen row    */
  long int PassCounter;                /* number of passes so far            */
  long int NumPasses;                  /* total number of array accesses     */
  double ElapsedTime;                  /* elapsed calculation time           */

  double AspectRatio;                  /* horizontal/vertical pel size       */
  int MaxColor;                        /* maximum color number               */
  int ColorShift;                      /* shift amount to assign colors      */
  unsigned int IterLimit;              /* max iterations at any point        */
  long unsigned int IterTotal;         /* total iterations for whole image   */
  long unsigned int InMSet;            /* total points in Mset               */

  DATUM PlaneCenter;                   /* the largest complex area ever used */
  DATUM PlaneSize;
  COMPLEX PlanePelSize;
  DATUM ImageCenter;                   /* complex plane currently in buffer  */
  DATUM ImageSize;
  COMPLEX PelSize;
  DATUM ScreenCenter;                  /* complex plane visible on screen    */
  DATUM ScreenSize;
  DATUM ResultCenter;                  /* result area for current array calc */
  DATUM ResultSize;
  int   ResultArea;                    /* 0 fills screen, 1 fills image      */
  DATUM ZoomCenter;                    /* complex plane in zoom box          */
  DATUM ZoomSize;
  DATUM ThisDot;                       /* current point being plotted        */
  char  ImageName[MAX_FLEN];           /* filespec for this name             */
};

union descblock {                      /* fixed-length descriptor            */
  struct descflds flds;
  unsigned int descarray[DESCRIPTORSIZE/2];
}


typedef union descblock DESCRIPTOR;


/*--- SERIAL.C                                                               */

#define THR  0x0000                    /* Transmitter Holding Register       */
#define RB   0x0000                    /* Receiver Buffer Register           */

#define DLL  0x0000                    /* Divisor Latch low                  */
#define DLH  0x0001                    /* Divisor Latch high                 */

#define IER  0x0001                    /* interrupt enable reg               */

#define IID  0x0002                    /* interrupt identification reg       */

#define LCR 0x0003                     /* line control  register port offset */
#define DLAB  0x80                     /*   divisor latch access bit         */
#define PSTK  0x20                     /*   force stuck parity               */
#define EPS   0x10                     /*   even parity select               */
#define PEN   0x08                     /*   parity enabled                   */
#define S8    0x03                     /*   8 data bits                      */

#define MCR 0x0004                     /* modem control register port offset */
#define DTR  0x01                      /*   DTR output                       */
#define RTS  0x02                      /*   RTS output                       */

#define LSR 0x0005                     /* line status   register port offset */
#define DR    0x01                     /*   receiver data ready              */
#define THRE  0x20                     /*   transmitter holding reg empty    */
#define TSRE  0x40                     /*   transmitter shift reg empty      */

#define MSR 0x0006                     /* modem status  register port offset */
#define DCTS  0x01                     /*   Delta CTS bit                    */
#define DDSR  0x02                     /*   Delta DSR bit                    */
#define CTS   0x10                     /*   CTS input                        */
#define DSR   0x20                     /*   DSR input                        */


/*---------------------------------------------------------------------------*/
/* Global variables                                                          */
/* All these, unfortunately, need to be shared by several routines           */

/*--- these describe the current image in enough detail to save & restore it */
/*    this is the basis of the zoom and file header info                     */

EXT long int PassesPerRow;             /* number of passes per screen row    */
EXT long int PassCounter;              /* number of passes so far            */
EXT long int NumPasses;                /* total number of array accesses     */
EXT double ElapsedTime;                /* elapsed calculation time           */

EXT double AspectRatio;                /* horizontal/vertical pel size       */
EXT int MaxColor;                      /* maximum color number               */
EXT unsigned int IterLimit;            /* max iterations at any point        */
EXT long unsigned int IterTotal;       /* total iterations for whole image   */
EXT int ColorShift;                    /* shift amount to assign colors      */
EXT long unsigned int InMSet;          /* total points in Mset               */

EXT DATUM PlaneCenter;                 /* the largest complex area ever used */
EXT DATUM PlaneSize;
EXT COMPLEX PlanePelSize;
EXT DATUM ImageCenter;                 /* complex plane currently in buffer  */
EXT DATUM ImageSize;
EXT COMPLEX PelSize;
EXT DATUM ResultCenter;                /* result area for current array calc */
EXT DATUM ResultSize;
EXT int   ResultArea                   /* 0 fills screen, 1 fills image      */
#ifdef INITIALIZE
                     = 0
#endif
;
EXT DATUM ScreenCenter;                /* complex plane visible on screen    */
EXT DATUM ScreenSize;
EXT DATUM ZoomCenter;                  /* complex plane in zoom box          */
EXT DATUM ZoomSize;
EXT DATUM ThisDot;                     /* current point being plotted        */


/*--- these are utility variables needed during image generation             */

EXT int NumEngines;                    /* number of array processor elements */
EXT int EngineRuns;                    /* total number of calculation runs   */
EXT enum {unknown=0,idle,ready,running} EngineState;
EXT unsigned int EngineValue[MAXENGINES];        /* result of latest calcs   */
EXT int Faking;                                  /* nonzero for simulation   */
EXT int TestPattern;                             /* array test pattern sel   */
EXT time_t StartTime;                  /* calculations started...            */
EXT time_t FinishTime;                 /*   ... and ended                    */

EXT unsigned char ExtAccum[2*NUMLEN];  /* EXTMATH -- extended accumulator    */

EXT int DrawRun;                       /* DRIVER -- main loop control        */
EXT int Changed;                       /* cleared when new image loaded      */

EXT jmp_buf ErrorEnv;                  /* global error return environment    */
EXT char   *ErrorMessage;              /* and final error message            */

EXT int  KeyChar;                      /* current keyboard character         */
EXT int  KeyCode;                      /*  ... and scan code                 */

EXT struct videoconfig vidmodes;       /* video hardware starting state      */

EXT int ScreenWidth;                   /* pels across visible screen         */
EXT int ScreenHeight;                  /* pels down visible screen           */

EXT unsigned char far * VSaveBuffer    /* save area for mode 13 screens      */
#ifdef INITIALIZE
                   = NULL
#endif
;

EXT double VIntsPerSec;                /* vertical interrupts per second     */

EXT int BackColor                      /* background fill color              */
#ifdef INITIALIZE
                   = 0
#endif
;

EXT unsigned int UseVidMode            /* desired video mode                 */
#ifdef INITIALIZE
                   = -1
#endif
;

EXT int CommPort                       /* serial port base address           */
#ifdef INITIALIZE
                  = 0x3F8
#endif
;

EXT long CommRate                      /* bits/second                        */
#ifdef INITIALIZE
                  = 28800
#endif
;


EXT unsigned int IterBase;             /* default iteration limit            */

EXT int ColorShiftBase;                /* default shift amount               */



